#ifndef COMPLEX_INCLUDE
#define COMPLEX_INCLUDE

#include <iostream>
#include <cmath>

template<class T=double>
class complex
{
public:
    complex(T, T);
    complex(T);

    complex(const complex<T>&);
    void copy(const complex<T>&);
    complex<T>* clone() const;
    void operator =(const complex<T>&);

    T real() const;
    T imag() const;

    complex<T>& operator +=(complex<T>);
    complex<T>& operator -=(complex<T>);
    complex<T>& operator *=(complex<T>);
    complex<T>& operator /=(complex<T>);
private:
    T re, im;
};

template<class T> complex<T> operator +(complex<T>, complex<T>);
template<class T> complex<T> operator -(complex<T>, complex<T>);
template<class T> complex<T> operator *(complex<T>, complex<T>);
template<class T> complex<T> operator /(complex<T>, complex<T>);

template<class T> complex<T> operator -(complex<T>);
template<class T> complex<T> operator +(complex<T>);

template<class T> bool operator ==(complex<T>,complex<T>);
template<class T> bool operator !=(complex<T>,complex<T>);

template<class T> std::ostream& operator <<(std::ostream &,const complex<T> &);
template<class T> std::istream& operator >>(std::istream &,complex<T> &);

template<class T> complex<T> polar(T rho, T theta);
template<class T> complex<T> conj(complex<T>);
template<class T> T abs(complex<T>);
template<class T> T arg(complex<T>);
template<class T> T norm(complex<T>);

template<class T> T real(complex<T>);
template<class T> T imag(complex<T>);

/*---------------------------------------------------------------------------------------*/
template<class T>
complex<T>::complex(T r=0.0,T i=0.0) : re(r), im(i)
{ }

template<class T>
complex<T>::complex(T r) : re(r), im(0.0)
{ }

template<class T>
complex<T>::complex(const complex<T> &c)
{
    copy(c);
}

template<class T>
void complex<T>::copy(const complex<T> &c)
{
    if (this == &c)
        return;
    this->re = c.re;
    this->im = c.im;
}

template<class T>
void complex<T>::operator =(const complex<T>& c)
{
    copy(c);
}

template<class T>
complex<T>* complex<T>::clone() const
{
    return new complex<T>(*this);
}

template<class T>
T complex<T>::real() const
{
    return re;
}

template<class T>
T complex<T>::imag() const
{
    return im;
}

template<class T>
complex<T>& complex<T>::operator +=(complex<T> a)
{
    re += a.re;
    im += a.im;
    return *this;
}

template<class T>
complex<T>& complex<T>::operator -=(complex<T> a)
{
    re -= a.re;
    im -= a.im;
    return *this;
}

template<class T>
complex<T>& complex<T>::operator *=(complex<T> a)
{
    re = re*a.re - im*a.im;
    im = re*a.im + im*a.re;
    return *this;
}

template<class T>
complex<T>& complex<T>::operator /=(complex<T> a)
{
    T div = a.re*a.re + a.im*a.im;
    re = re*a.re + im*a.im;
    re /= div;
    im = -re*a.im + im*a.re;
    im /= div;
    return *this;
}

template<class T>
complex<T> operator +(complex<T> a, complex<T> b)
{
    complex<T> r = a;
    return r += b;
}

template<class T>
complex<T> operator -(complex<T> a, complex<T> b)
{
    complex<T> r = a;
    return r -= b;
}

template<class T>
complex<T> operator *(complex<T> a, complex<T> b)
{
    complex<T> r = a;
    return r *= b;
}

template<class T>
complex<T> operator /(complex<T> a, complex<T> b)
{
    complex<T> r = a;
    return r /= b;
}

template<class T>
complex<T> operator -(complex<T> a)
{
    complex<T> r;
    return r -= a;
}

template<class T>
complex<T> operator +(complex<T> a)
{
    complex<T> r;
    return r += a;
}

template<class T>
bool operator ==(complex<T> a,complex<T> b)
{
    return a.real() == b.real() && a.imag() == b.imag();
}

template<class T>
bool operator !=(complex<T> a,complex<T> b)
{
    return a.real() != b.real() || a.imag() != b.imag();
}

template<class T>
complex<T> polar(T rho, T theta)
{
    complex<T> r(rho*std::cos(theta), rho*std::sin(theta));
    return r;
}

template<class T>
complex<T> conj(complex<T> a)
{
    complex<T> r(a.real(), -a.imag());
    return r;
}

template<class T>
T abs(complex<T> a)
{
    T n = a.real()*a.real() + a.imag()*a.imag();
    n = std::sqrt(n);
    return n;
}

template<class T>
T arg(complex<T> a)
{
    return std::atan2(a.imag(), a.real());
}

template<class T>
T norm(complex<T> a)
{
    return abs(a);
}

template<class T>
T real(complex<T> a)
{
    return a.real();
}

template<class T>
T imag(complex<T> a)
{
    return a.imag();
}

template<class T>
std::ostream& operator<<(std::ostream& out, const complex<T>& a)
{
    return out << '(' << a.real() << ',' << a.imag() << ')';
}

template<class T>
std::istream& operator >>(std::istream& in,complex<T>& a)
{
    T re = 0.0, im = 0.0;
    char ch = ' ';
    in >> ch;
    if ( ch == '(' )
    {
        in >> re >> ch;
        if ( ch == ',' )
        {
            in >> im >> ch;
            if ( ch == ')' )
            {
                a = complex<T>(re, im);
            }
            else
                in.setstate(std::ios_base::failbit);
        }
        else if ( ch == ')' )
        {
            a = re;
        }
        else
            in.setstate(std::ios_base::failbit);
    }
    else
    {
        in.putback(ch);
        in >> re;
        a = re;
    }
    return in;
}

#endif // COMPLEX_INCLUDE
